@@ -8,10 +8,6 @@ class FatLinkForm(forms.Form):
     type = forms.IntegerField(label=_("Type"), required=False)
 
 
-class FlatListForm(forms.Form):
-    flatlist = forms.CharField(widget=forms.Textarea)
-
-
 class ManualFatForm(forms.Form):
     character = forms.CharField(label=_("Character Name"), max_length=50)
     system = forms.CharField(label=_("System"), max_length=50)
